#include <u.h>
#include <libc.h>
#include <avl.h>
#include <bio.h>
#include <regexp.h>
#include "utils.h"
#include "muffet.h"

char* OUTDIR = "/lib/muffet/";


Resub match[MSZ];

char *
cannonizename(char * name){
	//fprint(2, " %s  - %d\n", name, strlen(name));
	int len = strlen(name);
	char * cannonname = (char*) malloc(len+1);
	for(int i = 0; name[i] != 0 ; ++i){
		if(name[i] == ' ')
			cannonname[i] = '_';
		else if(name[i] >= 'A' && name[i] <= 'Z')
			cannonname[i] = name[i] + 32;
		else
			cannonname[i] = name[i];
	}
	cannonname[len] = 0;

	return cannonname;
}
char *
decannonizename(char * name){
	//fprint(2, " %s  - %d\n", name, strlen(name));
	int len = strlen(name);
	char * cannonname = (char*) malloc(len+1);
	for(int i = 0; name[i] != 0 ; ++i){
		if(name[i] == '_')
			cannonname[i] = ' ';
		else if(name[i] >= 'a' && name[i] <= 'z')
			cannonname[i] = name[i] - 32;
		else
			cannonname[i] = name[i];
	}
	cannonname[len] = 0;

	return cannonname;
}

int
opentext(char * name){
	char *pathfmt = "%smedia/txt/%s.txt";
	char buf[100];
	int fd;
	name = cannonizename(name);
	snprint(buf, 100, pathfmt, OUTDIR, name);
	fd = open(buf, OREAD);
	free(name);
	return(fd);
}

int
openlinks(char * name){
	char *pathfmt = "%slinks/%s";
	char buf[100];
	int fd;
	name = cannonizename(name);
	snprint(buf, 100, pathfmt, OUTDIR, name);
	fd = create(buf, OWRITE, 0664);
	free(name);
	return(fd);
}

int bgexists(char * name){
	char * buf = (char*) malloc(100);
	snprint(buf, 100, "%smedia/bg/%s.png", OUTDIR, name);
	int t = open(buf, OREAD);
	close(t);
	return t;
}

int
opengem(char * name){
	char *pathfmt = "%sgemini/%s.gmi";
	char buf[100];
	int fd;
	name = cannonizename(name);
	snprint(buf, 100, pathfmt, OUTDIR, name);
	fd = create(buf, OWRITE, 0664);
	free(name);
	return(fd);
}

int
openhtml(char * name){
	char *pathfmt = "%shtml/%s.html";
	char buf[100];
	int fd;
	name = cannonizename(name);
	snprint(buf, 100, pathfmt, OUTDIR, name);
	fd = create(buf, OWRITE, 0664);
	free(name);
	return(fd);
}

char * 
copymatch(int i)
{
	int len;
	char * s_match;
	if(match[i].ep && match[i].sp && match[i].ep > match[i].sp){
		len = (long long)match[i].ep - (long long)match[i].sp;
		s_match = malloc(len+1);
		strncpy(s_match,match[i].sp, len);
		s_match[len] = 0;
		for(int i = len -1 ; s_match[i] == ' ' || s_match[i] == '	'; i--)
			s_match[i] = 0;
		return s_match;
	}
	return nil;
}

int
mcuistrcmp(char* a, char* b){
	int ret;
	char * at = malloc(strlen(a) + 1);
	char * bt = malloc(strlen(b) + 1);
	for(int i = strlen(a) ; i >= 0; i--){
		at[i] = ( (a[i] == '_') ? ' ': ( (a[i] > 'Z' && a[i] < '{' ) ? (a[i]-' '):a[i]) );
	}
	for(int i = strlen(b) ; i >= 0; i--){
		bt[i] = ( (b[i] == '_') ? ' ': ( (b[i] > 'Z' && b[i] < '{' ) ? (b[i]-' '):b[i]) );
	}
	ret = strcmp(at,bt);
	free(at);
	free(bt);
	return ret;
}

int mncmp( Avl * i , Avl * j){
	muffetnode *I = (muffetnode*) i;
	muffetnode *J = (muffetnode*) j;
	return mcuistrcmp(I->name, J->name);
}
int mccmp( Avl * i , Avl * j){
	muffetchild *I = (muffetchild*) i;
	muffetchild *J = (muffetchild*) j;
	return mcuistrcmp(I->name, J->name);
}
int mbcmp( Avl * i , Avl * j){
	muffetbody *I = (muffetbody*) i;
	muffetbody *J = (muffetbody*) j;
	return I->index - J->index;
}
int mlcmp( Avl * i , Avl * j){
	muffetlink *I = (muffetlink*) i;
	muffetlink *J = (muffetlink*) j;
	return I->index - J->index;
}
int mircmp( Avl * i , Avl * j){
	muffet_inc_ref *I = (muffet_inc_ref*) i;
	muffet_inc_ref *J = (muffet_inc_ref*) j;
	return mcuistrcmp(I->name, J->name);
}
int morcmp( Avl * i , Avl * j){
	muffet_out_ref *I = (muffet_out_ref*) i;
	muffet_out_ref *J = (muffet_out_ref*) j;
	return I->index - J->index;
}
int mtcmp( Avl * i , Avl * j){
	muffettag *I = (muffettag*) i;
	muffettag *J = (muffettag*) j;
	return mcuistrcmp(I->name, J->name);
};

void
walk(Avltree * tree, Biobuf * in, int out, void(*action)(Avl *, Biobuf * in, int out, int dir), int dir){
	Avl * dot;
	if(dir){dot = avlmax(tree);} else {dot = avlmin(tree);}
	do{
		if(dot)
			action(dot, in, out, dir);
	}while (dot = dir?avlprev(dot):avlnext(dot));
}
